@@ -25,7 +25,8 @@ class GrapheneUnshardedMeshSource(UnshardedLegacyPrecomputedMeshSource):
 
   def compute_filename(self, label):
     layer_id = self.meta.meta.decode_layer_id(label)
-    chunk_block_shape = 2 * Vec(*self.meta.meta.mesh_chunk_size)
+    chunk_block_shape = Vec(*self.meta.meta.mesh_chunk_size, dtype=np.int64)
+    chunk_block_shape *= np.int64(self.meta.meta.fan_out ** max(0, layer_id - 2))
     start = self.meta.meta.decode_chunk_position(label)
     start *= chunk_block_shape
     bbx = Bbox(start, start + chunk_block_shape)
@@ -43,26 +44,31 @@ class GrapheneUnshardedMeshSource(UnshardedLegacyPrecomputedMeshSource):
     ]
 
     cloudpath = self.meta.join(self.meta.cloudpath, self.meta.mesh_path)
-    return CloudFiles(cloudpath).exists(filenames)
+    return CloudFiles(cloudpath, secrets=self.config.secrets).exists(filenames)
 
   def get_fragment_labels(self, segid, lod=0, level=2, bbox=None, bypass=False):
     if bypass:
       return [ segid ]
 
-    manifest = self.fetch_manifest(segid, lod, level, bbox, return_segids=True)
+    manifest = self.fetch_manifest(segid, lod, level, bbox, return_segids=True, verify=False)
     return manifest["seg_ids"]
 
   def get_fragment_filenames(self, segid, lod=0, level=2, bbox=None, bypass=False):
     if bypass:
       return [ self.compute_filename(segid) ]
 
-    manifest = self.fetch_manifest(segid, lod, level, bbox)
+    manifest = self.fetch_manifest(segid, lod, level, bbox, verify=True)
     return manifest["fragments"]
 
-  def fetch_manifest(self, segid, lod=0, level=2, bbox=None, return_segids=False):
+  def fetch_manifest(self, segid, lod=0, level=2, bbox=None, return_segids=False, verify=True):
+    """
+    verify: the server calls exists and returns byte ranges. this is intended 
+      to take advantage of the lower latency on the server side, but can be a
+      bottleneck.
+    """
     # TODO: add lod to endpoint
     query_d = {
-      'verify': True,
+      'verify': bool(verify),
     }
     if return_segids:
       query_d['return_seg_ids'] = 1
@@ -138,7 +144,7 @@ class GrapheneUnshardedMeshSource(UnshardedLegacyPrecomputedMeshSource):
       self, segids, 
       remove_duplicate_vertices=False, 
       fuse=False, bounding_box=None,
-      bypass=False, use_byte_offsets=True,
+      bypass=False, use_byte_offsets=False,
       deduplicate_chunk_boundaries=True,
       allow_missing=False,
     ):
