@@ -1,4 +1,4 @@
-import collections
+from collections.abc import Iterable
 import json
 import os
 import posixpath
@@ -20,7 +20,7 @@ from ...lib import (
   colorize, red, mkdir, 
   Vec, Bbox, jsonify, 
 )
-from ...paths import strict_extract, ascloudpath
+from ...paths import strict_extract, ascloudpath, to_https_protocol
 
 def downscale(size, factor_in_mip, roundingfn):
   smaller = Vec(*size, dtype=np.float32) / Vec(*factor_in_mip)
@@ -37,17 +37,19 @@ class PrecomputedMetadata(object):
   read and write Precomputed data types.
   """
   def __init__(
-    self, cloudpath, cache=None, 
+    self, cloudpath, config=None, cache=None, 
     info=None, provenance=None, 
-    max_redirects=10
+    max_redirects=10, use_https=False
   ):
     self.path = strict_extract(cloudpath)
     self.cache = cache
     if self.cache:
       self.cache.meta = self
+    self.config = config
     self.info = None
 
     self.redirected_from = []
+    self.use_https = use_https
 
     if info is None:
       self.refresh_info(max_redirects=max_redirects)
@@ -192,7 +194,7 @@ class PrecomputedMetadata(object):
 
     Returns: dict
     """
-    info = CloudFiles(self.cloudpath).get_json('info')
+    info = CloudFiles(self.cloudpath, secrets=self.config.secrets).get_json('info')
 
     if info is None:
       raise exceptions.InfoUnavailableError(
@@ -241,6 +243,9 @@ class PrecomputedMetadata(object):
         break
 
       path = strict_extract(info['redirect'])
+      if self.use_https:
+        path = to_https_protocol(path)
+
       if path == self.path:
         break 
       elif path in visited:
@@ -300,7 +305,7 @@ Hops:
       separators=(',', ': ')
     )
     # use put instead of put_json to preserve formatting
-    cf = CloudFiles(self.cloudpath)
+    cf = CloudFiles(self.cloudpath, secrets=self.config.secrets)
     cf.put(
       'info', infojson,
       cache_control='no-cache',
@@ -360,7 +365,7 @@ Hops:
 
     Returns: dict
     """
-    cf = CloudFiles(self.cloudpath)
+    cf = CloudFiles(self.cloudpath, secrets=self.config.secrets)
     provfile = cf.get('provenance')
     if provfile:
       provfile = provfile.decode('utf-8')
@@ -403,7 +408,7 @@ Hops:
     )
 
     # need to use put vs put_json to preserve formatting
-    cf = CloudFiles(self.cloudpath)
+    cf = CloudFiles(self.cloudpath, secrets=self.config.secrets)
     cf.put(
       'provenance', prov, 
       cache_control='no-cache', 
@@ -545,7 +550,7 @@ Hops:
     return Vec(*self.info['scales'][mip]['resolution'])
 
   def to_mip(self, mip):
-    mip = list(mip) if isinstance(mip, collections.Iterable) else int(mip)
+    mip = list(mip) if isinstance(mip, Iterable) else int(mip)
     try:
       if isinstance(mip, list):  # mip specified by voxel resolution
         return next(
