@@ -17,7 +17,7 @@ class ShardedPrecomputedSkeletonSource(object):
     if self.meta.spatial_index:
       mip = self.meta.mip or 0
       self.spatial_index = CachedSpatialIndex(
-        self.cache,
+        self.cache, self.config,
         cloudpath=self.meta.layerpath, 
         bounds=self.meta.meta.bounds(mip) * self.meta.meta.resolution(mip),
         chunk_size=self.meta.info['spatial_index']['chunk_size'],
