@@ -48,13 +48,25 @@ class NumpyEncoder(json.JSONEncoder):
       return float(obj)
     return json.JSONEncoder.default(self, obj)
 
+def nvl(*args):
+  for arg in args:
+    if arg is not None:
+      return arg
+  return None
+
 def first(lst):
   if isinstance(lst, types.GeneratorType):
-    return next(lst)
+    try:
+      return next(lst)
+    except StopIteration:
+      return None
   try:
     return lst[0]
   except TypeError:
-    return next(iter(lst))
+    try:
+      return next(iter(lst))
+    except StopIteration:
+      return None
 
 def sip(iterable, block_size):
   """Sips a fixed size from the iterable."""
@@ -683,6 +695,11 @@ class Bbox(object):
     result.maxpt = np.round(result.maxpt / chunk_size) * chunk_size
     return (result + offset).astype(self.dtype)
 
+  def num_chunks(self, chunk_size):
+    """Computes the number of chunks inside this bbox for a given chunk size."""
+    Nfn = lambda i: math.ceil((self.maxpt[i] - self.minpt[i]) / chunk_size[i])
+    return reduce(operator.mul, map(Nfn, range(len(self.minpt))))
+
   def contains(self, point):
     """
     Tests if a point on or within a bounding box.
